package ch.zhaw.projekt2.turnierverwaltung;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class TournamentDecorator implements IsObservable{
    private Tournament tournament;
    private FileIO fileIO;
    private List<IsObserver> listener = new ArrayList<>();

    public TournamentDecorator(FileIO fileIO){
        setFileIO(fileIO);
        addListener(new IsObserver() {
            @Override
            public void update() {
                if(tournament != null){
                    saveTournament();
                }
            }
        });
    }

    public void setFileIO(FileIO fileIO) {
        this.fileIO = fileIO;
    }

    public void setTournament(Tournament tournament) {
        this.tournament = tournament;
    }

    public Tournament getTournament() {
        return tournament;
    }

    @Override
    public void addListener(IsObserver observer) {
        listener.add(observer);
    }

    @Override
    public void removeListener(IsObserver observer) {
        listener.remove(observer);
    }

    public void saveTournament(){
        try {
            fileIO.saveTournament(tournament);
        } catch (IOException e) {
            e.printStackTrace(); //TODO handle and logging
        }
    }

    public void createTournament(String name, Tournament.Type type){
        if(fileIO.tournamentExists(name)){
            System.out.println("Tournament with same name exists already.");
            return; //TODO handle and logging
        }
        try {
            Tournament tournament = new Tournament(name, type);
            fileIO.saveTournament(tournament);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
        } catch (Tournament.InvalidTypeException e) {
            e.printStackTrace(); //TODO handle and logging
        } catch (IOException e) {
            e.printStackTrace(); //TODO handle and logging
        }
    }

    public void deleteTournament(FileIO.TournamentFile tournamentFile){
        try {
            fileIO.deleteTournament(tournamentFile);
            informListener();
        } catch (IOException e) {
            e.printStackTrace();  //TODO handle and logging
        }
    }

    public void createNewGameSchedule() {
        try {
            tournament.createGameSchedule();
            informListener();
        } catch (Tournament.NumberOfParticipantInvalidException e) {
            e.printStackTrace();
        }
    }

    public void savePlayer(String firstName, String name, String phoneNumber, String dateOfBirth){
        try {
            tournament.addParticipant(new Player(firstName, name, phoneNumber, dateOfBirth));
            informListener();
        } catch (Tournament.ParticipantExistsException e) {
            e.printStackTrace(); //TODO handle and logging
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
        } catch (Person.InvalidPhoneNumberException e) {
            e.printStackTrace(); //TODO handle and logging
        }
    }

    public void deleteParticipant(Participant participant){
        try {
            tournament.removeParticipant(participant);
            informListener();
        } catch (Tournament.ParticipantNotExistsException e) {
            e.printStackTrace(); //TODO handle and logging
        }
    }

    public void savePlace(String name){
        try {
            tournament.addPlace(new Place(name));
            informListener();
        } catch (Tournament.PlaceExistsException e) {
            e.printStackTrace(); //TODO handle and logging
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
        }
    }

    public void deletePlace(Place place){
        try {
            tournament.removePlace(place);
            informListener();
        } catch (Tournament.PlaceNotExistsException e) {
            e.printStackTrace(); //TODO handle and logging
        }
    }

    public void informListener() {
        for(IsObserver observer : listener) {
            observer.update();
        }
    }
}
